#include <iostream>
#include <fstream>
#include <string>
#include <regex>
#include "JGP.h"
#include "Stop.h"

using namespace std;

JGP* JGP::instance = nullptr;

std::ostream& operator<<(std::ostream& ot, const JGP& jgp) {
	ot << "\t Informacije o javnom gradskom prevozu: " << endl << endl;
	for (pair<string, Line*> pair : jgp.lines) {
		cout << *pair.second;
	}
	return ot;
}

void JGP::deleteInstance()
{
	JGP& jgp = *instance;
	for (auto iter : jgp.lines) delete iter.second;
	for (auto iter : jgp.stops) delete iter.second;
}

void JGP::parse(string directory)
{
	ifstream file(directory + "linije.txt");
	string textLine;
	regex rx("([^;]*);([^;]*);([^;]*);");


	while (getline(file, textLine)) {
		smatch result;
		if (regex_match(textLine, result, rx)) {
			string name = result.str(1);
			string from = result.str(2);
			string to = result.str(3);

			Line* line = new Line(name, from, to);
			lines[name] = line;
		}
	}

	for (pair<string, Line*> pair : lines) {
		pair.second->parse(directory + "linija_" +pair.second->getName() + ".txt");
	}
}

void JGP::addStop(Stop* s)
{
	stops[s->getNumber()] = s;
}

map<pair<Line*, Line*>, int> JGP::allPairs(int min) const
{
	map<pair<Line*, Line*>, int> map;

	for_each(stops.begin(), stops.end(), [&map](pair<int,Stop*> ps) {
		for (Line* l1 : *ps.second) {
			for (Line* l2 : *ps.second) {
				if (l1->getName() < l2->getName()) {
					map[pair<Line*,Line*>(l1, l2)]++;
				}
			}
		}
	});

	for (auto iter = map.begin(); iter != map.end();) {
		if (iter->second < min) iter = map.erase(iter);
		else iter++;
	}
	return map;
}

vector<Stop*> JGP::stopsWithMostLines() const
{
	vector<Stop*> res;

	using PT = pair<int, Stop*>;
	int max = max_element(stops.begin(), stops.end(), 
		[](PT a, PT b) {return a.second->numberOfLines() < b.second->numberOfLines(); })->second->numberOfLines();

	for_each(stops.begin(), stops.end(), [max, &res](PT a) {if (a.second->numberOfLines() == max) res.push_back(a.second); });
	return res;
}

void JGP::exportGraph(string fileName)
{
	ofstream file(fileName);

	file << "graph" << endl << "[" << endl;
	for (pair<int, Stop*> pair : stops) {
		Stop* stop = pair.second;

		file << "\tnode\n\t[\n\t\tid " << stop->getNumber() << "\n\t\tlabel \"" << stop->getNumber() << " - " << stop->getName() << "\"\n\t]\n";
	}

	set<pair<Stop*, Stop*>> edges;

	for (pair<string, Line*> pair : lines) {
		Line* line = pair.second;

		Stop* prev = nullptr;
		for (Stop* stop : *line) {
			if (prev != nullptr && edges.find(make_pair(prev, stop)) == edges.end()) {
				edges.insert(make_pair(prev, stop));
				file << "\tedge\n\t[\n\t\tsource " << prev->getNumber() << "\n\t\ttarget \"" << stop->getNumber() << "\"\n\t]\n";
			}
			prev = stop;
		}
	}

	file << "]";
	file.flush();
	file.close();
}
