#include <algorithm>
#include <fstream>
#include <regex>
#include <set>

#include "Line.h"
#include "Stop.h"
#include "JGP.h"


ostream& operator<<(ostream& ot, const Line& l) {
	ot << endl << "\t" << l.name << "\t" << l.from << "  -  " << l.to << endl << endl;
	for_each(l.begin(), l.end(), [&ot](Stop * s) { ot << *s; });
	return ot;
}

void Line::parse(string fileName)
{
	ifstream file(fileName);
	string s;
	regex rx("\\[\"([^\\[]*)\\[#([0-9]*)\\];([^\"]*)\", ([^,]*),([^,]*),([^\\]]*)\\];.*");

	JGP& jgp = JGP::getInstance();

	while (getline(file, s)) {
		smatch result;
		if (regex_match(s, result, rx)) {

			string name = result.str(1);
			int number = atoi(result.str(2).c_str());
			string slines = result.str(3);
			vector<Line*> lines;
			double x = atof(result.str(4).c_str()), y = atof(result.str(5).c_str());
			int zone = atoi(result.str(6).c_str());;

			regex rxl("([^ ]+) \\|");
			sregex_iterator begin(slines.begin(), slines.end(), rxl);
			sregex_iterator end;

			while (begin != end) {
				smatch sm = *begin;
				string lineName = sm.str(1);
				lines.push_back(jgp[lineName]);
				begin++;
			}


			Stop* stop = jgp[number];
			if (stop == nullptr) {
				stop = new Stop(name, number, lines, x, y, zone);
				jgp.addStop(stop);
			}
			stops.push_back(stop);
		}

		
	}
}

set<Line*> Line::commonStopsWith() const
{
	set<Line*> result;
	for (Stop* s : stops) {
		for (Line* l : *s) {
			if (this != l) result.insert(l);
		}
	}
	return result;
}

vector<Stop*> Line::edgeStops() const
{
	vector<Stop*> result;

	
	
	result.push_back(*max_element(stops.begin(), stops.end(), [](Stop * a, Stop * b) { return a->getX() < b->getX(); }));

	result.push_back(*max_element(stops.begin(), stops.end(), [](Stop * a, Stop * b) { return a->getY() < b->getY(); }));

	result.push_back(*max_element(stops.begin(), stops.end(), [](Stop * a, Stop * b) { return a->getX() > b->getX(); }));

	result.push_back(*max_element(stops.begin(), stops.end(), [](Stop * a, Stop * b) { return a->getY() > b->getY(); }));

	return result;
}

pair<Line*, int> Line::mostCommonStopsWith() const
{
	map<Line*,int> map;
	for (Stop* s : stops) {
		for (Line* l : *s) {
			if (this != l) map[l]++;
		}
	}
	using PT = pair<Line*, int>;
	return *max_element(map.begin(), map.end(), [](PT a, PT b) {return a.second < b.second; });
}
